#ifndef FLEX_ACTIONS_HEADER
#define FLEX_ACTIONS_HEADER

#include "../../backend/support/shared.h"

/**
 * Se definen los diferentes IDs de cada token disponible para el scanner Flex.
 */
typedef enum TokenID {

	// Por defecto, el valor "0" hace fallar el analizador sintáctico.
	UNKNOWN = 0,

	// Código de error de Bison, que permite abortar el escaneo de lexemas cuando
	// se presente un patrón desconocido. El número "257" coincide con el valor
	// que Bison le otorga por defecto, pero además permite que el resto de
	// tokens continúen desde el valor "258" lo que permite proteger los IDs
	// internos que Bison reserva para crear "tokens literales":
	YYUNDEF = 257,

	// Operadores aritméticos.
	ADD,
	SUB,
	MUL,
	DIV,
	POW,

	// Brackets.
	OPEN_PARENTHESIS,
	CLOSE_PARENTHESIS,
	OPEN_BRACES,
	CLOSE_BRACES,
	OPEN_BRACKETS,
	CLOSE_BRACKETS,

	FUNCTION,
	IF,
	ELSE,
	FOR,
	PRINT,
	SEMICOLON,

	COMPOSITE,
	DERIVATIVE,
	INTEGRATE,
	BETWEEN,
	AND,
	EVALUATE,
	IN,

	COLON,
	COMMA,
	GREATER,
	GREATER_EQUAL,
	LESSER,
	LESSER_EQUAL,
	ASSIGN,
	EQUAL,

	ADD_ASSIGN,
	INCREMENT,
	SUB_ASSIGN,
	DECREMENT,
	QUOTE,
	
	// Tipos de dato.
	INTEGER,
	DOUBLE,
	STRING
} TokenID;

/**
 * Se definen las acciones a ejecutar sobre cada patrón hallado mediante el
 * analizador léxico Flex. Este analizador solo puede identificar
 * construcciones regulares, ya que utiliza un autómata finito determinístico
 * (a.k.a. DFA), como mecanismo de escaneo y reconocimiento.
 */

TokenID IntegerPatternAction(const char * lexeme);

TokenID DoublePatternAction(const char * lexeme);

void IgnoredPatternAction(const char * lexeme);

TokenID UnknownPatternAction(const char * lexeme);

TokenID	StringPatternAction(const char * lexeme);

#endif
